CHATS_TO_FETCH = 20
CHATS_TO_FETCH__DOC = (
    "The number of most recent chats to fetch on startup. "
    "Getting all chats might hit rate limiting and possibly account lock. "
    "Please report if you try with high values and don't hit any problem!"
)
